@@ -6,7 +6,10 @@
 
 #include <Rdefines.h>
 
-/* These same values are hard-coded into the R source. Don't change them! */
+/* These same values are hard-coded into the R source. Don't change them! 
+ * Note that they are not the same as values defined in the netcdf
+ * library headers, since I don't want my R code to depend on those 
+ */
 #define R_NC_TYPE_SHORT  1
 #define R_NC_TYPE_INT    2
 #define R_NC_TYPE_FLOAT  3
@@ -14,6 +17,14 @@
 #define R_NC_TYPE_TEXT   5
 #define R_NC_TYPE_BYTE   6
 
+/* New types in netcdf version 4 */
+#define R_NC_TYPE_UBYTE		7
+#define R_NC_TYPE_USHORT	8
+#define R_NC_TYPE_UINT		9
+#define R_NC_TYPE_INT64		10
+#define R_NC_TYPE_UINT64	11
+#define R_NC_TYPE_STRING	12
+
 /*********************************************************************
  * Converts from type "nc_type" to an integer as defined in the beginning 
  * of this file.  We do NOT use the raw nc_type integers because then the
@@ -34,21 +45,36 @@ int R_nc4_nctype_to_Rtypecode( nc_type nct )
 		return(R_NC_TYPE_DOUBLE);
 	else if( nct == NC_BYTE )
 		return(R_NC_TYPE_BYTE);
+
+	else if( nct == NC_UBYTE )
+		return(R_NC_TYPE_UBYTE);
+	else if( nct == NC_USHORT )
+		return(R_NC_TYPE_USHORT);
+	else if( nct == NC_UINT )
+		return(R_NC_TYPE_UINT);
+	else if( nct == NC_INT64 )
+		return(R_NC_TYPE_INT64);
+	else if( nct == NC_UINT64 )
+		return(R_NC_TYPE_UINT64);
+	else if( nct == NC_STRING )
+		return(R_NC_TYPE_STRING);
 	else
 		return(-1);
 }
 
 /*********************************************************************/
-/* Returns a vector of dim sizes for the variable */
-void R_nc4_varsize( int *ncid, int *varid, int *varsize, int *retval )
+/* Returns a vector of dim sizes for the variable.
+ * 'retval' is 0 for no error, or -1 for an error.
+ */
+void R_nc4_varsize( int *ncid, int *varid, int *ndims, int *varsize, int *retval )
 {
-	int 	i, err, ndims, dimid[NC_MAX_DIMS];
+	int 	i, err, dimid[NC_MAX_DIMS];
 	size_t	dimlen;
 
 	*retval = 0;
 
 	/* Get ndims */
-	err = nc_inq_varndims( *ncid, *varid, &ndims );
+	err = nc_inq_varndims( *ncid, *varid, ndims );
 	if( err != NC_NOERR ) {
 		Rprintf( "Error in R_nc4_varsize on nc_inq_varndims call: %s\n", 
 			nc_strerror(err) );
@@ -66,7 +92,7 @@ void R_nc4_varsize( int *ncid, int *varid, int *varsize, int *retval )
 		}
 
 	/* Get size of each dim */
-	for( i=0; i<ndims; i++ ) {
+	for( i=0; i<(*ndims); i++ ) {
 		err = nc_inq_dimlen( *ncid, dimid[i], &dimlen );
 		if( err != NC_NOERR ) {
 			Rprintf( "Error in R_nc4_varsize on nc_inq_dimlen call: %s\n",
@@ -194,19 +220,77 @@ void R_nc4_get_vara_double( int *ncid, int *varid, int *start,
 			nc_strerror(*retval) );
 		Rprintf( "Var: %s  Ndims: %d   Start: ", vn, ndims );
 		for( i=0; i<ndims; i++ ) {
-			Rprintf( "%u", s_start[i] );
+			Rprintf( "%u", (unsigned int)s_start[i] );
 			if( i < ndims-1 )
 				Rprintf( "," );
 			}
 		Rprintf( "Count: " );
 		for( i=0; i<ndims; i++ ) {
-			Rprintf( "%u", s_count[i] );
+			Rprintf( "%u", (unsigned int)s_count[i] );
 			if( i < ndims-1 )
 				Rprintf( "," );
 			}
 		}
 }
 
+/*********************************************************************/
+void R_nc4_get_vara_double_fixmiss( int *ncid, int *varid, int *start, 
+	int *count, int *imvstate, double *missval, double *data, int *retval )
+{
+	int	i, err, ndims;
+	size_t	s_start[MAX_NC_DIMS], s_count[MAX_NC_DIMS], tot_size, ii;
+	char	vn[2048];
+	double	mvtol;
+
+	err = nc_inq_varndims(*ncid, *varid, &ndims );
+	if( err != NC_NOERR ) 
+		Rprintf( "Error in R_nc4_get_vara_double while getting ndims: %s\n", 
+			nc_strerror(*retval) );
+
+	tot_size = 1L;
+	for( i=0; i<ndims; i++ ) {
+		s_start[i] = (size_t)start[i];
+		s_count[i] = (size_t)count[i];
+		tot_size   = tot_size * s_count[i];
+		}
+		
+	*retval = nc_get_vara_double(*ncid, *varid, s_start, s_count, data );
+	if( *retval != NC_NOERR ) {
+		nc_inq_varname( *ncid, *varid, vn );
+		Rprintf( "Error in R_nc4_get_vara_double: %s\n", 
+			nc_strerror(*retval) );
+		Rprintf( "Var: %s  Ndims: %d   Start: ", vn, ndims );
+		for( i=0; i<ndims; i++ ) {
+			Rprintf( "%u", (unsigned int)s_start[i] );
+			if( i < ndims-1 )
+				Rprintf( "," );
+			}
+		Rprintf( "Count: " );
+		for( i=0; i<ndims; i++ ) {
+			Rprintf( "%u", (unsigned int)s_count[i] );
+			if( i < ndims-1 )
+				Rprintf( "," );
+			}
+		}
+
+	/* We only do this for imvstate==2, which is a valid, non-NA
+	 * missing value. If the variable has NO missing value
+	 * (imvstate==0) or a NA missing value (imvstate==1) then
+	 * we skip this block.
+	 */
+	if( *imvstate == 2 ) {
+		if( *missval == 0.0 ) 
+			mvtol = 1.e-10;	/* arbitrary -- what are you supposed to use?? */
+		else
+			mvtol = fabs( *missval ) * 1.e-5;
+
+		for( ii=0L; ii<tot_size; ii++ ) {
+			if( fabs( data[ii] - *missval ) < mvtol )
+				data[ii] = NA_REAL;
+			}
+		}
+}
+
 /*********************************************************************/
 /* byte_style is 1 for signed, 2 for unsigned
  */
@@ -237,13 +321,13 @@ void R_nc4_get_vara_int( int *ncid, int *varid, int *start,
 			nc_strerror(*retval) );
 		Rprintf( "Var: %s  Ndims: %d   Start: ", vn, ndims );
 		for( i=0; i<ndims; i++ ) {
-			Rprintf( "%u", s_start[i] );
+			Rprintf( "%u", (unsigned int)s_start[i] );
 			if( i < ndims-1 )
 				Rprintf( "," );
 			}
 		Rprintf( "Count: " );
 		for( i=0; i<ndims; i++ ) {
-			Rprintf( "%u", s_count[i] );
+			Rprintf( "%u", (unsigned int)s_count[i] );
 			if( i < ndims-1 )
 				Rprintf( "," );
 			}
@@ -294,13 +378,13 @@ void R_nc4_get_vara_text( int *ncid, int *varid, int *start,
 			nc_strerror(*retval) );
 		Rprintf( "Var: %s  Ndims: %d   Start: ", vn, ndims );
 		for( i=0; i<ndims; i++ ) {
-			Rprintf( "%u", s_start[i] );
+			Rprintf( "%u", (unsigned int)s_start[i] );
 			if( i < ndims-1 )
 				Rprintf( "," );
 			}
 		Rprintf( "Count: " );
 		for( i=0; i<ndims; i++ ) {
-			Rprintf( "%u", s_count[i] );
+			Rprintf( "%u", (unsigned int)s_count[i] );
 			if( i < ndims-1 )
 				Rprintf( "," );
 			}
@@ -313,6 +397,24 @@ void R_nc4_get_vara_text( int *ncid, int *varid, int *start,
 		}
 }
 
+/*********************************************************************/
+/* Returns -1 if the dim is not found in the file. Otherwise,
+ * returns the dimension's length
+ */
+void R_nc4_inq_dimlen( int *ncid, char **dimname, int *dimlen )
+{
+	int err, dimid;
+	err = nc_inq_dimid(*ncid, dimname[0], &dimid );
+	if( err != NC_NOERR ) {
+		*dimlen = -1;
+		return;
+		}
+
+	size_t st_dimlen;
+	err = nc_inq_dimlen( *ncid, dimid, &st_dimlen );
+	*dimlen = (int)st_dimlen;
+}
+
 /*********************************************************************/
 /* Returns -1 if the dim is not found in the file */
 void R_nc4_inq_dimid( int *ncid, char **dimname, int *dimid )
@@ -501,9 +603,63 @@ nc_type R_nc4_ttc_to_nctype( int type_to_create )
 	if( type_to_create == 6 )
 		return( NC_BYTE );
 
-	Rprintf("Error, R_nc4_ttc_to_nctype passed unknown value: %d\n",
+	error("Error, R_nc4_ttc_to_nctype passed unknown value: %d\n",
 		type_to_create );
-	exit(-1);
+	return( NC_BYTE ); /* -Wall */
+}
+
+/*********************************************************************/
+/* NOTE that NA's are handled through this vector.  According to the 
+ * docs as of 2010-11-02, NA's have the value MIN_INT */
+void R_nc4_put_att_logical( int *ncid, int *varid, char **attname, 
+		int *type_to_create, int *natts, int *attribute, int *retval )
+{
+	int	R_NA_val;
+	float	C_NA_val_f;
+	double 	C_NA_val_d;
+
+	/* Rprintf( "in R_nc4_put_att_logical with val=%d\n", *attribute ); */
+	nc_type ttc;
+	ttc = R_nc4_ttc_to_nctype( *type_to_create );
+
+	R_NA_val = -2147483648;		/* From R docs */
+
+	if( *attribute == R_NA_val ) {
+		/* Rprintf( "PUTTING a NA -- float \n" ); */
+		/* Put a NA */
+		if( ttc == NC_FLOAT ) {
+			C_NA_val_f = 0./0.;
+			*retval = nc_put_att_float(*ncid, *varid, attname[0], 
+				ttc, *natts, &C_NA_val_f );
+			if( *retval != NC_NOERR ) 
+				Rprintf( "Error in R_nc4_put_att_logical: %s\n", 
+					nc_strerror(*retval) );
+			}
+
+		else if( ttc == NC_DOUBLE ) {
+			/* Rprintf( "PUTTING a NA -- double \n" ); */
+			C_NA_val_d = 0./0.;
+			*retval = nc_put_att_double(*ncid, *varid, attname[0], 
+				ttc, *natts, &C_NA_val_d );
+			if( *retval != NC_NOERR ) 
+				Rprintf( "Error in R_nc4_put_att_logical: %s\n", 
+					nc_strerror(*retval) );
+			}
+		else
+			{
+			Rprintf( "Error in R_nc4_put_att_logical: asked to put a NA value, but the variable's type is not a double or float, which are the only two types that have a defined NaN value\n" );
+			*retval = -1;
+			return;
+			}
+
+		}
+	else
+		*retval = nc_put_att_int(*ncid, *varid, attname[0], 
+			ttc, *natts, attribute );
+
+	if( *retval != NC_NOERR ) 
+		Rprintf( "Error in R_nc4_put_att_logical: %s\n", 
+			nc_strerror(*retval) );
 }
 
 /*********************************************************************/
@@ -621,6 +777,7 @@ void R_nc4_get_att_text( int *ncid, int *varid, char **attname,
 		attribute[0] );
 	/* append a NULL */
 	if( *retval != NC_NOERR ) {
+		Rprintf( "R_nc4_get_att_text: error encountered on call to nc_get_att_text: %s\n", nc_strerror(*retval) );
 		strcpy( attribute[0], "\0" );
 		return;
 		}
@@ -632,6 +789,71 @@ void R_nc4_get_att_text( int *ncid, int *varid, char **attname,
 	attribute[0][attlen] = '\0';
 }
 
+/*********************************************************************/
+/* With strings, we explicitly allocate space for them here.
+ * Note that the string array is returned as the function return value; the
+ * error code is passed in and out as an argument.
+ */
+SEXP R_nc4_get_att_string( SEXP sx_ncid, SEXP sx_varid, SEXP sx_attname, SEXP sx_attlen, SEXP sx_ierr_returned )
+{
+	int	ncid, varid, attlen, ierr, i, slen;
+	char	**strings;
+	const char *attname = CHAR(STRING_ELT(sx_attname,0));
+	char	*tstr;
+	SEXP	sx_string, sx_ret_str_array;
+
+	INTEGER(sx_ierr_returned)[0] = 0;	/* initialize to "no error" */
+
+	ncid   = INTEGER(sx_ncid  )[0];
+	varid  = INTEGER(sx_varid )[0];
+	attlen = INTEGER(sx_attlen)[0];	/* for strings, this is the NUMBER of strings, not any string's length */
+
+	if( attlen < 1 ) {
+		Rf_error( "Error, in call to R_nc4_get_att_string, number of strings (attlen) must be >= 1\n" );
+		INTEGER(sx_ierr_returned)[0] = -1;
+		return( R_NilValue );
+		}
+
+	/* Make space in C to hold array of pointers to strings */
+	strings = (char **)R_alloc( attlen, sizeof(char *) );
+
+	/* Read the strings from the netcdf file. Note that the netcdf library
+	 * call allocates space for the strings themselves
+	 */
+	if( (ierr = nc_get_att_string( ncid, varid, attname, strings )) != NC_NOERR ) {
+		Rf_error( "Error, in call to R_nc4_get_att_string, failed to get the strings:\n" );
+		Rf_error( nc_strerror( ierr ) );
+		INTEGER(sx_ierr_returned)[0] = -1;
+		return( R_NilValue );
+		}
+
+	/* Initialize our return value
+	 */
+	sx_ret_str_array = PROTECT( NEW_CHARACTER( attlen ));	/* make array with "attlen" strings */
+
+	/* Copy from our C array to an R array that we make. We do this so 
+	 * that we can free the C netcdf-library allocated space, which
+	 * otherwise would never be freed
+	 */
+	for( i=0; i<attlen; i++ ) {
+		slen = strlen( strings[i] );
+		tstr = R_alloc( slen+1, sizeof(char) );
+		strncpy(tstr, strings[i], slen);
+		tstr[slen] = '\0';
+		sx_string = PROTECT( mkChar(tstr) );
+
+		SET_STRING_ELT( sx_ret_str_array, i, sx_string );
+		}
+
+	/* Free C memory allocated by netcdf library
+	 */
+	nc_free_string( attlen, strings );
+
+	UNPROTECT(attlen+1);
+
+	return( sx_ret_str_array );
+}
+
 /*********************************************************************/
 void R_nc4_put_vara_double( int *ncid, int *varid, int *start,
 	int *count, double *data, int *retval )
@@ -1208,7 +1430,7 @@ SEXP R_nc4_blankstring(SEXP size)
 */
 SEXP R_nc4_grpname(SEXP sx_root_id, SEXP sx_ierr_retval)
 {
-	int	i, root_id, ierr;
+	int	root_id, ierr;
 	size_t	nchar;
 	char	*str;
 	SEXP	sx_retval, sx_string;
@@ -1266,7 +1488,7 @@ SEXP R_nc4_grpname(SEXP sx_root_id, SEXP sx_ierr_retval)
 */
 SEXP R_nc4_inq_format(SEXP sx_root_id, SEXP sx_ierr_retval)
 {
-	int	i, root_id, ierr;
+	int	root_id, ierr;
 	int	iformat, iretval;
 	SEXP	sx_retval;
 
